{*******************************************************************************
Unit ThreadController - This unit contains code to start and stop the threads.
*******************************************************************************}
unit threadcontroller;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, SimpleThread;

type
  { ThreadController }

  //A class to manage the threads.
  TThreadController = Class(TObject)
    protected
      FThreadArray: array of TSimpleThread; //Array of all threads created
    public
      destructor Destroy; override;
      procedure CreateOneThread;
      procedure TerminateOneThread;
      procedure TerminateAllThreads;
      function GetNumberOfThreads: Integer;
    end;


implementation

{ ThreadController }

destructor TThreadController.Destroy;
begin
  //Terminate all running threads to free the memory.
  if GetNumberOfThreads > 0 then
  begin
    TerminateAllThreads;
  end;

  inherited Destroy;
end;

procedure TThreadController.CreateOneThread;
begin
  //Extend the array by one.
  setlength(FThreadArray, length(FThreadArray) + 1);

  //Create and start the thread.
  FThreadArray[high(FThreadArray)] := TSimpleThread.Create(False);

  //Check if creation failed.
  if Assigned(FThreadArray[high(FThreadArray)].FatalException) then
    raise FThreadArray[high(FThreadArray)].FatalException;
end;

//Terminates the last thread in the array and shorten the array by one,
//so we have the correct number of running threads.
procedure TThreadController.TerminateOneThread;
begin
  if length(FThreadArray) > 0 then
  begin
    FThreadArray[high(FThreadArray)].Terminate;
    setlength(FThreadArray, length(FThreadArray) - 1);
  end;
end;

procedure TThreadController.TerminateAllThreads;
begin
  while length(FThreadArray) > 0 do
    TerminateOneThread;
end;

function TThreadController.GetNumberOfThreads: Integer;
begin
  Result:= length(FThreadArray);
end;

end.

